local RegisteringHandler = FlexBar2.OO.Mixin 
{
	"RegisterEvent", -- Custom RegisterEvent function, Registers the event with AceEvent if it ain't already. It is table-based, read its code for details ;)
	"UnregisterEvent",
	"RegisterScript", -- Custom version of the SetScript function. Very similar to my custom RegisterEvent function except it is for SetScript stuff :)
	"UnregisterScript",
	"UnregisterAll",
};

function RegisteringHandler:RegisterEvent(Event, MethodName) -- Note: Events are not shared with the groupmembers as they already get them by registering the event anyway
	FlexBar2:Debug("Registering " , Event, " with " , MethodName);
	if(type(self[MethodName]) ~= "function") then
		return false
	end
	if(not FlexBar2:IsEventRegistered(Event)) then
		FlexBar2[Event] = function(...)
			for Button , EventTable in pairs(FlexBar2.EventTable[Event]) do
				for _, MethodName in pairs(EventTable) do
					Button[MethodName](Button, Event, ...);
				end
			end
		end
		FlexBar2:RegisterEvent(Event);
	end
	if(type(FlexBar2.EventTable[Event]) ~= "table") then
		FlexBar2.EventTable[Event] = {};
	end
	if(type(FlexBar2.EventTable[Event][self]) ~= "table") then
		FlexBar2.EventTable[Event][self] = {};
	end
	table.insert(FlexBar2.EventTable[Event][self], MethodName);
end

function RegisteringHandler:UnregisterEvent(Event, MethodName)
	if(MethodName) then
		return FlexBar2:RemoveByValue(FlexBar2.EventTable[Event][self], MethodName);
	else
		for n,v in pairs(FlexBar2.EventTable[Event][self]) do
			table.remove(FlexBar2.EventTable[Event][self], n);
		end
	end
end


function RegisteringHandler:RegisterScript(Trigger, MethodName) -- Widget scripts are sent to the group handler if there is one available, if not they are not shared
	if(type(self.ScriptTable) ~= "table") then
		self.ScriptTable = {};
	end
	if(type(self.Frame:GetScript(Trigger)) ~= "function") then
		self.Frame:SetScript(Trigger, function(...)
			for _ , ScriptName in ipairs(self.ScriptTable[Trigger]) do
				if(Trigger ~= "OnUpdate") then FlexBar2:Debug("Triggering ", ScriptName, " For Handler: ", Trigger); end
				if(type(self.Group) == "table" and type(self.GroupHandler) == "table" and type(self.GroupHandler[ScriptName]) == "function") then
					self.GroupHandler[ScriptName](self, Trigger, ...);
				else
					self[ScriptName](self, Trigger, ...);
				end
			end
		end);
	end
	if(type(self.ScriptTable[Trigger]) ~= "table") then
		self.ScriptTable[Trigger] = {};
	end
	table.insert(self.ScriptTable[Trigger], MethodName);
end

function RegisteringHandler:UnregisterScript(Trigger, MethodName)
	if(type(self.ScriptTable) ~= "table") then
		self.ScriptTable = {};
	end
	if(MethodName) then
		return FlexBar2:RemoveByValue(self.ScriptTable[Trigger], MethodName);
	else
		for n,v in pairs(self.ScriptTable[Trigger]) do
			table.remove(self.ScriptTable[Trigger], n);
		end
	end
end


FlexBar2:RegisterHandler(RegisteringHandler);
