--[[
Notes on grouping: this code on itself has no use, it is used in the ButtonRegisteringHandler.lua file to share widget events between group members.
Regular blizzard events are not shared as all buttons get them already anyway
--]]
local Group = {};
Group.Groups = {};
Group.Handlers = {};

FlexBar2:AddStartupScript(function()
	local Buttons = FlexBar2.Buttons;
	for GroupName, GroupTable in pairs(FlexBar2.db.profile.Groups) do
		Group:New(GroupName);
		for _, MemberName in ipairs(GroupTable) do
			Group:Add(GroupName, Buttons[MemberName]);
		end
	end
end);

function Group:New(GroupName, Save)
	if(type(GroupName) ~= "string") then
		return;
	end
	if(type(self.Groups[GroupName]) ~= "table") then
		self.Groups[GroupName] = {};
	else
		return;
	end
	if(Save == true) then
		if(type(FlexBar2.db.profile.Groups) ~= "table") then
			FlexBar2.db.profile.Groups = {};
		end
		FlexBar2.db.profile.Groups[GroupName] = {};
	end
end

function Group:Disband(GroupName, Save)
	if(type(GroupName) ~= "string") then
		return;
	end
	if(type(self.Groups[GroupName]) == "table") then
		for _, Member in ipairs(FlexBar2:CopyTable(self.Groups[GroupName], true)) do
			self:Remove(GroupName, Member, Save);
		end
		self.Groups[GroupName] = nil;
		if(Save == true) then
			FlexBar2.db.profile.Groups[GroupName] = nil;
		end
	end
end

function Group:Add(GroupName, Member, Save)
	if(type(GroupName) ~= "string" or type(self.Groups[GroupName]) ~= "table") then
		return;
	end
	table.insert(self.Groups[GroupName], Member);
	local Group = self.Groups[GroupName];
	local GroupHandler = self.Handlers;
	for _, GroupMember in ipairs (self.Groups[GroupName]) do
		if(type(GroupMember.Group) ~= "table") then
			GroupMember.Group = Group;
			GroupMember.GroupHandler = GroupHandler;
		end
	end
	if(Save == true) then
		if(Member.Button ~= nil) then
			table.insert(FlexBar2.db.profile.Groups[GroupName], Member.Button);
		end
	end
end

function Group:Remove(GroupName, Member, Save)
	if(type(GroupName) ~= "string" or type(self.Groups[GroupName]) ~= "table" or type(Member) ~= "table") then
		return;
	end
	FlexBar2:RemoveByValue(self.Groups[GroupName], Member);
	Member.Group = nil;
	Member.GroupHandler = nil;
	if(Save == true) then
		FlexBar2:RemoveByValue(FlexBar2.db.profile.Groups[GroupName], Member.Button);
	end
end

function Group:RegisterHandler(Name, Handler)
	if(type(self.Handlers[Name]) ~= "function") then
		self.Handlers[Name] = Handler;
	end
end

FlexBar2.Group = Group;
